unit TwenUnit;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls, Buttons, ComCtrls, WavEngU, WaveObj, MMSystem, Gauges,
	MMIOFile;

type
	TMainForm = class(TForm)
		InputEdit: TEdit;
		InputLabel: TLabel;
		OutputEdit: TEdit;
		OutputLabel: TLabel;
		OpenBitBtn: TBitBtn;
		OpenDialog: TOpenDialog;
		SaveDialog: TSaveDialog;
		SaveBitBtn: TBitBtn;
		InputFormatRadioGroup: TRadioGroup;
		ProcessBitBtn: TBitBtn;
		OutputFormatRadioGroup: TRadioGroup;
		GaugePanel: TPanel;
		ProgressGauge: TGauge;
		SRateComboBox: TComboBox;
		StopBitBtn: TBitBtn;
		SRStaticText: TStaticText;
		procedure OpenBitBtnClick(Sender: TObject);
		procedure SaveBitBtnClick(Sender: TObject);
		procedure InputEditExit(Sender: TObject);
		procedure OutputEditExit(Sender: TObject);
		procedure ProcessBitBtnClick(Sender: TObject);
		procedure SRateComboBoxExit(Sender: TObject);
		procedure StopBitBtnClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure SRateComboBoxClick(Sender: TObject);
	private
		{ Private-Deklarationen }
		InFile, OutFile: TMMIOFile;
		InDataPos, OutDataPos, SampleRate: LongInt;
		Samples: Cardinal;
		InWH, OutWH: TWaveHeader;
		MotorolaFormat, converting, deleteafter: Boolean;
		function ReduceRes: Boolean;
		procedure SetControls(Status: Boolean);
	public
		{ Public-Deklarationen }
	end;
	PCM20b = record
		Left1, Right1, Left2, Right2: SmallInt;
		LowL1, LowR1, LowL2, LowR2: Byte;
	end;

var
	MainForm: TMainForm;
	WaveData: PCM20b;
	InFileName, OutFileName: PChar;

implementation

{$R *.DFM}

procedure TMainForm.OpenBitBtnClick(Sender: TObject);
begin
  OpenDialog.FileName := InputEdit.Text;
  if OpenDialog.Execute
	then begin
    InputEdit.Text := OpenDialog.FileName;
    if LowerCase(ExtractFileExt(InputEdit.Text)) = '.wav'
    then OutputEdit.Text := ChangeFileExt(InputEdit.Text, '_new')+'.wav'
    else OutputEdit.Text := ChangeFileExt(InputEdit.Text, '.wav')
  end;
	if OpenDialog.FilterIndex = 1
  then InputFormatRadioGroup.ItemIndex := 0;
  if OpenDialog.FilterIndex = 2
  then InputFormatRadioGroup.ItemIndex := 1;
  ProcessBitBtn.Enabled := (InputEdit.Text > '') and (OutputEdit.Text > '')
end;

procedure TMainForm.SaveBitBtnClick(Sender: TObject);
begin
  SaveDialog.FileName := OutputEdit.Text;
	if SaveDialog.Execute
  then OutputEdit.Text := SaveDialog.FileName;
  ProcessBitBtn.Enabled := (InputEdit.Text > '') and (OutputEdit.Text > '')
end;

procedure TMainForm.InputEditExit(Sender: TObject);
begin
	ProcessBitBtn.Enabled := (InputEdit.Text > '') and (OutputEdit.Text > '')
end;

procedure TMainForm.OutputEditExit(Sender: TObject);
begin
  ProcessBitBtn.Enabled := (InputEdit.Text > '') and (OutputEdit.Text > '')
end;

function TMainForm.ReduceRes: Boolean;
begin
	Result := (MessageDlg('Too many samples to be handled in a'^M
											 +'RIFF WAVE file at 24 bit resolution;'^M
											 +'Okay to reduce resolution down to'^M
											 +'16 bits and proceed?', mtConfirmation,
											 [mbYes, mbNo], 0) = mrYes);
	if Result
	then OutputFormatRadioGroup.ItemIndex := 0
end;

procedure TMainForm.ProcessBitBtnClick(Sender: TObject);
var
  B: Byte;
  FileProgress: Cardinal;
begin
	if InputEdit.Text = OutputEdit.Text
  then begin
    MessageDlg('Output = input file name', mtError, [mbOK], 0);
    Exit
  end;
  if not FileExists(InputEdit.Text)
	then begin
    MessageDlg('Input file does not exist', mtError, [mbOK], 0);
    Exit
	end;
	if FileExists(OutputEdit.Text)
  and (MessageDlg('Output file exists - Overwrite?', mtConfirmation, [mbYes, mbNo], 0) <> mrYes)
	then Exit;
	InFileName := StrNew(PChar(InputEdit.Text));
	InFile := OpenSampleFile(InFileName);
	if InFile = NIL
	then begin
		MessageDlg('Unable to open input file', mtError, [mbOK], 0);
		Exit
	end
	else begin
		InFile.SetBufferSize($8000);
		InDataPos := ReadWaveHeader(InFile, InWH, Samples);
		MotorolaFormat := (InDataPos = 0);
		ResetFilePointer(InFile);
		MoveFilePointer(InFile, InDataPos);
		OutFileName := StrNew(PChar(OutputEdit.Text));
		OutFile := CreateSampleFile(OutFileName, true);
		if OutFile = NIL
		then begin
			MessageDlg('Unable to create output file', mtError, [mbOK], 0);
			Exit
		end
		else begin
		  OutFile.SetBufferSize($8000);
			deleteafter := false;
			converting := true;
			OutDataPos := 0;
			InFile.SeekEOF;
			Samples := (InFile.FilePos-InDataPos) div 10;
			ProgressGauge.MinValue := 0;
			ProgressGauge.Progress := 0;
			ProgressGauge.MaxValue := Samples;
			InFile.SeekTo(InDataPos);
			if (OutputFormatRadioGroup.ItemIndex = 0)
			or (Samples < (($FFFFFFFF div 12) - 64))
			or ReduceRes
			then begin
				with OutWH do begin
					with fmt do begin
						ChunkID := chFmt;
						Size := SizeOf(fmt)-8;
						wFormatTag := WAVE_FORMAT_PCM;
						wChannels := 2;
						dwSamplesPerSec := SampleRate;
						wBlockAlign := wChannels*(2+OutputFormatRadioGroup.ItemIndex);
						dwAvgBytesPerSec := dwSamplesPerSec*wBlockAlign;
						wBitsPerSample := 4*(4+OutputFormatRadioGroup.ItemIndex);
						cbSize := 0
					end;
					with data do begin
						ChunkID := chData;
						Size := Samples*fmt.wBlockAlign*fmt.wChannels
					end;
					with RIFF do begin
						RIFF_ID := rmRIFF;
						SubType := rtWAVE;
						Size := 38+data.Size
					end
				end;
				FileProgress := 0;
				ProcessBitBtn.Enabled := false;
				Screen.Cursor := crHourGlass;
				SetControls(false);
				try
					OutFile.Write(@OutWH, 46);
					try
						ProcessBitBtn.Visible := False;
						StopBitBtn.Visible := True;
						while (InFile.Read(@WaveData.Left1, 2) = 2)
						and (InFile.Read(@WaveData.Right1, 2) = 2)
						and (InFile.Read(@WaveData.Left2, 2) = 2)
						and (InFile.Read(@WaveData.Right2, 2) = 2)
						and (InFile.Read(@WaveData.LowL1, 1) = 1)
						and (InFile.Read(@WaveData.LowR1, 1) = 1)
						and converting
						do begin
							if MotorolaFormat
							then begin
								WaveData.Left1 := Swap(WaveData.Left1);
								WaveData.Right1 := Swap(WaveData.Right1);
								WaveData.Left2 := Swap(WaveData.Left2);
								WaveData.Right2 := Swap(WaveData.Right2)
							end
							else begin
								B := WaveData.LowR1;
								WaveData.LowR1 := WaveData.LowL1;
								WaveData.LowL1 := B
							end;
							Application.ProcessMessages;
							if OutputFormatRadioGroup.ItemIndex = 0
							then begin
								OutFile.Write(@WaveData.Left1, 2);
								OutFile.Write(@WaveData.Right1, 2);
								OutFile.Write(@WaveData.Left2, 2);
								OutFile.Write(@WaveData.Right2, 2);
							end
							else begin
								WaveData.LowL2 := (WaveData.LowL1 and $0F) shl 4;
								WaveData.LowL1 := WaveData.LowL1 and $F0;
								WaveData.LowR2 := (WaveData.LowR1 and $0F) shl 4;
								WaveData.LowR1 := WaveData.LowR1 and $F0;
								OutFile.Write(@WaveData.LowL1, 1);
								OutFile.Write(@WaveData.Left1, 2);
								OutFile.Write(@WaveData.LowR1, 1);
								OutFile.Write(@WaveData.Right1, 2);
								OutFile.Write(@WaveData.LowL2, 1);
								OutFile.Write(@WaveData.Left2, 2);
								OutFile.Write(@WaveData.LowR2, 1);
								OutFile.Write(@WaveData.Right2, 2);
							end;
							Inc(FileProgress);
							ProgressGauge.Progress := FileProgress;
							Application.ProcessMessages;
						end
					except
						on E:Exception do MessageDlg(E.Message, mtError, [mbOK], 0)
					end
				finally
					StopBitBtn.Visible := False;
					ProcessBitBtn.Visible := True;
					OutFile.Close;
					if deleteafter then OutFile.Erase;
					OutFile.Free;
					InFile.Close;
					InFile.Free;
					SetControls(true);
					Screen.Cursor := crDefault;
					ProcessBitBtn.Enabled := true;
				end
			end;
			converting := false
    end
  end
end;

procedure TMainForm.SRateComboBoxExit(Sender: TObject);
begin
	try
		SampleRate := StrToInt(SRateComboBox.Text)
	except
		on E:EConvertError do begin
			MessageDlg('Invalid numeric format.', mtError, [mbOK], 0);
			SRateComboBox.ItemIndex := 0;
		end
	end
end;

procedure TMainForm.StopBitBtnClick(Sender: TObject);
const
	Msg: String = 'You stopped the conversion.'#13+
								#13+
								'Press "Abort" to abort the conversion and delete the output file.'#13+
								'Press "Retry" to resume the conversion.'#13+
								'Press "Ignore" to stop the conversion but leave the output file.';

begin
	case MessageDlg(Msg, mtConfirmation, mbAbortRetryIgnore, 0) of
		mrAbort: begin
			converting := False;
			deleteafter := True
		end;
		mrRetry: begin
			converting := True;
			deleteafter := False
		end;
		mrIgnore: begin
			converting := False;
			deleteafter := False
		end
	end
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  SampleRate := 48000;
end;

procedure TMainForm.SRateComboBoxClick(Sender: TObject);
begin
	try
		SampleRate := StrToInt(SRateComboBox.Text)
	except
		on E:EConvertError do begin
			MessageDlg('Invalid numeric format.', mtError, [mbOK], 0);
			SRateComboBox.ItemIndex := 0;
		end
	end
end;

procedure TMainForm.SetControls(Status: Boolean);
begin
	OpenBitBtn.Enabled := Status;
	SaveBitBtn.Enabled := Status;
	InputEdit.Enabled := Status;
	OutputEdit.Enabled := Status;
	InputFormatRadioGroup.Enabled := Status;
	OutputFormatRadioGroup.Enabled := Status;
	SRateComboBox.Enabled := Status;
end;

end.

